﻿#region Using Statements
using System;
using RhinoXNA.RhinoGraphics.Rhino2D;
using Microsoft.Xna.Framework;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Graphics;
using RhinoXNA;
#endregion

namespace AsteroidsGame.Game_Objects
{
    /// <summary>
    /// Asteroid class
    /// </summary>
    public class Asteroid : SpriteEntity
    {
        #region Private Variables
        // Movement variables
        private Vector2 _movementDir;
        private int _movementSpeed;

        // Rotation amount
        private int _rotationMultiplier;

        // Starting health amount (shouldn't change)
        private int _maxHealth;
        #endregion

        #region Public Properties
        /// <summary>
        /// Returns the current health of this asteroid
        /// </summary>
        public int Health { get; set; }

        /// <summary>
        /// Returns the starting health of this asteroid
        /// </summary>
        public int MaxHealth
        {
            get { return _maxHealth; }
        }

        /// <summary>
        /// The current movement direction
        /// </summary>
        public Vector2 MovementDirection
        {
            get { return _movementDir; }
            set { _movementDir = value; }
        }

        /// <summary>
        /// Store last collided asteroid so they don't stick together
        /// </summary>
        public Asteroid LastCollision { get; set; }
        #endregion

        #region Public Methods
        /// <summary>
        /// Instantiates a new object of this type
        /// </summary>
        /// <param name="parameters">Parameters defining this object</param>
        /// <param name="isBrandNew">Whether the asteroid is new or a child of another</param>
        public Asteroid(List<String> parameters, bool isBrandNew)
            : base(parameters)
        {
            if (isBrandNew)
            {
                do
                {
                    float randomDirection = RhinoXNAAdapter.Instance.RandomInt(RhinoXNAAdapter.Instance.RandomInt(10, 30), RhinoXNAAdapter.Instance.RandomInt(50, 100));
                    if (randomDirection % 2 == 0)
                    {
                        randomDirection = 1;
                    }
                    else
                    {
                        randomDirection = -1;
                    }

                    // Variables to randomize a side and movement angle
                    float randomAngle = RhinoXNAAdapter.Instance.RandomInt(10, 170);
                    float randomSide = RhinoXNAAdapter.Instance.RandomInt(1, 1001);
                    Matrix transform = Matrix.Identity;
                    _movementDir = new Vector2();

                    // Randomize a side and movement diretion
                    if (randomSide > 500)
                    {
                        if (randomSide > 750)
                        {
                            // Right side of screen
                            this.Position = new Vector2(RhinoXNAAdapter.Instance.RandomInt(RhinoXNAAdapter.Instance.SCREEN_SAFE.Right + 300, RhinoXNAAdapter.Instance.SCREEN_SAFE.Right + 500),
                                RhinoXNAAdapter.Instance.RandomInt(RhinoXNAAdapter.Instance.SCREEN_SAFE.Top, RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom));

                            transform = Matrix.CreateTranslation(new Vector3(this.Position, 1.0f))
                                * Matrix.CreateRotationY(MathHelper.ToRadians(90 + randomAngle));
                        }
                        else
                        {
                            // Left side of screen
                            this.Position = new Vector2(RhinoXNAAdapter.Instance.RandomInt(RhinoXNAAdapter.Instance.SCREEN_SAFE.Left - 500, RhinoXNAAdapter.Instance.SCREEN_SAFE.Left - 300),
                                RhinoXNAAdapter.Instance.RandomInt(RhinoXNAAdapter.Instance.SCREEN_SAFE.Top, RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom));

                            transform = Matrix.CreateTranslation(new Vector3(this.Position, 1.0f))
                                * Matrix.CreateRotationY(MathHelper.ToRadians(270 + randomAngle));
                        }

                    }
                    else
                    {
                        if (randomSide > 250)
                        {
                            // Bottom of screen
                            this.Position = new Vector2(RhinoXNAAdapter.Instance.RandomInt(RhinoXNAAdapter.Instance.SCREEN_SAFE.Left, RhinoXNAAdapter.Instance.SCREEN_SAFE.Right),
                                RhinoXNAAdapter.Instance.RandomInt(RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom + 300, RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom + 500));

                            transform = Matrix.CreateTranslation(new Vector3(this.Position, 1.0f))
                                * Matrix.CreateRotationY(MathHelper.ToRadians(randomAngle));
                        }
                        else
                        {
                            // Top of screen
                            this.Position = new Vector2(RhinoXNAAdapter.Instance.RandomInt(RhinoXNAAdapter.Instance.SCREEN_SAFE.Left, RhinoXNAAdapter.Instance.SCREEN_SAFE.Right),
                                RhinoXNAAdapter.Instance.RandomInt(RhinoXNAAdapter.Instance.SCREEN_SAFE.Top - 500, RhinoXNAAdapter.Instance.SCREEN_SAFE.Top - 300));

                            transform = Matrix.CreateTranslation(new Vector3(this.Position, 1.0f))
                                * Matrix.CreateRotationY(MathHelper.ToRadians(360 - randomAngle));
                        }
                    }

                    _movementDir = Vector2.Transform(this.Position, transform);
                    _movementDir.Normalize();
                    this.Health = _maxHealth = 4;
                } while (AsteroidFactory.Instance.Asteroids.Exists(delegate(Asteroid a) { return a.BasicBoundingBox.Intersects(this.BasicBoundingBox); }));
            }
            else
            {
                this.Health = _maxHealth = Convert.ToInt32(parameters[14]);
                _movementDir = new Vector2(float.Parse(parameters[15]), float.Parse(parameters[16]));
                this.Position = new Vector2(this.Position.X + (Convert.ToInt32(parameters[17]) * (this.Width / 2)),
                    this.Position.Y);
            }



            _rotationMultiplier = RhinoXNAAdapter.Instance.RandomInt(20, RhinoXNAAdapter.Instance.RandomInt(100, 150));
            _movementSpeed = RhinoXNAAdapter.Instance.RandomInt(4, 9);
        }
        #endregion

        #region Protected Methods
        /// <summary>
        /// Animate implementation for this class
        /// </summary>
        /// <param name="gameTime">Current gametime</param>
        protected override void Animate(Microsoft.Xna.Framework.GameTime gameTime)
        {
            this.Rotation += MathHelper.ToRadians(((float)gameTime.ElapsedGameTime.TotalSeconds)*_rotationMultiplier);
            this.Position = new Vector2(this.Position.X + (_movementSpeed * _movementDir.X), this.Position.Y + (_movementSpeed * _movementDir.Y));

            bool swappedHorizontal = false;
            bool swappedVertical = false;

            if ((this.Position.Y + this.Height/2) < 0 && !swappedVertical)
            {
                this.Position = new Vector2(this.Position.X,
                    RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom + this.Height / 2);
                swappedVertical = true;
            } 
            else if ((this.Position.Y - this.Height/2) > RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom && !swappedVertical)
            {
                this.Position = new Vector2(this.Position.X,
                    RhinoXNAAdapter.Instance.SCREEN_SAFE.Top - this.Height / 2);
                swappedVertical = true;
            }

            if ((this.Position.X - this.Width/2) > RhinoXNAAdapter.Instance.SCREEN_SAFE.Right && !swappedHorizontal)
            {
                this.Position = new Vector2(
                    RhinoXNAAdapter.Instance.SCREEN_SAFE.Left - this.Width / 2,
                    this.Position.Y);
                swappedHorizontal = true;
            }
            else if ((this.Position.X + this.Width/2) < RhinoXNAAdapter.Instance.SCREEN_SAFE.Left && !swappedHorizontal)
            {
                this.Position = new Vector2(
                    RhinoXNAAdapter.Instance.SCREEN_SAFE.Right + this.Width / 2,
                    this.Position.Y);
                swappedHorizontal = true;
            }
        }
        #endregion
    }
}
