﻿#region Using Statements
using System;
using System.Collections.Generic;
using RhinoXNA;
using Microsoft.Xna.Framework;
using RhinoXNA.RhinoGameObjects;
#endregion

namespace AsteroidsGame.Game_Objects
{
    public class Bullet : DrawableProjectile
    {
        public Vector2 MovementDirection { get; set; }

        private float _life = 0.0f;

        public Bullet(List<String> parameters) 
            : base (parameters)
        {
            this.MovementSpeed = 8.0f;
            this.MovementDirection = new Vector2(float.Parse(parameters[14]), float.Parse(parameters[15]));
        }

        protected override void Animate(Microsoft.Xna.Framework.GameTime gameTime)
        {
            _life += gameTime.ElapsedGameTime.Milliseconds;

            this.Position = new Vector2(
                this.Position.X + (this.MovementDirection.X * this.MovementSpeed),
                this.Position.Y + (this.MovementDirection.Y * this.MovementSpeed));

            for (int i = 0; i < AsteroidFactory.Instance.Asteroids.Count; i++)
            {
                Asteroid asteroid = AsteroidFactory.Instance.Asteroids[i];
                if (this.BasicBoundingBox.Intersects(asteroid.BasicBoundingBox))
                {
                    asteroid.Health--;

                    if (asteroid.Health <= 0)
                    {
                        AsteroidFactory.Instance.SplitAsteroid(asteroid);
                    }

                    RhinoEntityManager.Instance.RemoveEntity(this.Handle);
                }
            }

            bool swappedHorizontal = false;
            bool swappedVertical = false;

            if ((this.Position.Y + this.Height / 2) < 0 && !swappedVertical)
            {
                this.Position = new Vector2(this.Position.X,
                    RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom + this.Height / 2);
                swappedVertical = true;
            }
            else if ((this.Position.Y - this.Height / 2) > RhinoXNAAdapter.Instance.SCREEN_SAFE.Bottom && !swappedVertical)
            {
                this.Position = new Vector2(this.Position.X,
                    RhinoXNAAdapter.Instance.SCREEN_SAFE.Top - this.Height / 2);
                swappedVertical = true;
            }

            if ((this.Position.X - this.Width / 2) > RhinoXNAAdapter.Instance.SCREEN_SAFE.Right && !swappedHorizontal)
            {
                this.Position = new Vector2(
                    RhinoXNAAdapter.Instance.SCREEN_SAFE.Left - this.Width / 2,
                    this.Position.Y);
                swappedHorizontal = true;
            }
            else if ((this.Position.X + this.Width / 2) < RhinoXNAAdapter.Instance.SCREEN_SAFE.Left && !swappedHorizontal)
            {
                this.Position = new Vector2(
                    RhinoXNAAdapter.Instance.SCREEN_SAFE.Right + this.Width / 2,
                    this.Position.Y);
                swappedHorizontal = true;
            }

            if (_life >= 2500.0f)
            {
                RhinoEntityManager.Instance.RemoveEntity(this.Handle);
            }
        }
    }
}
