﻿#region Using Statements
using RhinoXNA;
using RhinoXNA.RhinoUtilities;
using Microsoft.Xna.Framework.Content;
using AsteroidsGame.Menus;
using Microsoft.Xna.Framework.Input;
using AsteroidsGame.Base_Objects;
using AsteroidsGame.Game_Objects;
using RhinoXNA.RhinoGraphics.Rhino2D.RhinoHUD;
using System;
using System.Collections.Generic;
using RhinoXNA.RhinoGameObjects;
using Microsoft.Xna.Framework;
#endregion

namespace AsteroidsGame.Levels
{
    /// <summary>
    /// Space level implementation
    /// </summary>
    public class SpaceLevel : RhinoLevel
    {
        #region Private Variables
        // Wave variables
        private int _waveCount = 0;
        private bool _waveStart = false;
        private int _waveStartCount = 0;
        #endregion

        #region Public Methods
        /// <summary>
        /// Instantiates a new object of this class
        /// </summary>
        public SpaceLevel()
            : base()
        {
            RhinoEntityManager.Instance.AddEntities(RhinoLoader.GetGraphics("LoadFiles\\ContentLoad.properties", "LoadFiles\\Space.lfi"));

            RhinoHUDManager.Instance.AddEntity(new HUDText(new List<String>()
            {
                "HUDText", 
                "Fonts\\HUDFont", 
                "HUD-Lives", 
                "Lives:", 
                "L200", "T50", 
                "0", 
                "1.0", "1.0", 
                "0.6", 
                "true"
            }));

            _inputBinding = new InputBinding(Keyboard.GetState(), Mouse.GetState());

            GenerateNewWave();
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Generates a new asteroid wave
        /// </summary>
        private void GenerateNewWave()
        {
            _waveStart = true;
            _waveCount++;

            RhinoEntityManager.Instance.AddEntity(new HUDText(new List<String>()
            {
                "HUDText", 
                "Fonts\\WaveFont", 
                "WaveText", 
                "Wave: " + _waveCount.ToString(), 
                "CENTER", "C-200", 
                "0", 
                "1.0", "1.0", 
                "0.6", 
                "true"
            }));
        }
        #endregion

        #region Protected Methods
        /// <summary>
        /// Update implementation for the space level
        /// </summary>
        /// <param name="gameTime">Current gametime</param>
        protected override void UpdateImpl(Microsoft.Xna.Framework.GameTime gameTime)
        {
            if (AsteroidFactory.Instance.Asteroids.Count <= 0 && _waveStart == false)
            {
                GenerateNewWave();
            }

            if (_waveStart == true)
            {
                _waveStartCount += gameTime.ElapsedGameTime.Milliseconds;

                if (_waveStartCount >= 2500)
                {
                    _waveStart = false;
                    _waveStartCount = 0;

                    RhinoEntityManager.Instance.RemoveEntity("WaveText");

                    AsteroidFactory.Instance.GenerateDefaultAsteroids(_waveCount);
                }
            }

            AsteroidFactory.Instance.Update(gameTime);

            if (_inputBinding.CheckKeyDown(Keyboard.GetState(), Keys.Escape))
            {
                RhinoXNAAdapter.Instance.AddNewMenu(new PauseMenu());
            }

            _inputBinding.Update();
        }

        /// <summary>
        /// Render implementation for this level
        /// </summary>
        protected override void RenderImpl()
        {
            AsteroidFactory.Instance.Render();
        }
        #endregion
    }
}
