﻿#region Using Statements
using System;
using RhinoXNA.RhinoGraphics.Rhino2D;
using AsteroidsGame.Menus;
using Microsoft.Xna.Framework.Graphics;
using RhinoXNA.RhinoUtilities;
using RhinoXNA;
using Microsoft.Xna.Framework.Input;
using AsteroidsGame.Menus;
using AsteroidsGame.Base_Objects;
using Microsoft.Xna.Framework;
#endregion

namespace AsteroidsGame.Menus
{
    public enum Transition
    {
        Left,
        Right,
        None,
        Center
    }

    public class MainMenu : RhinoMenu
    {
        private int _focusIndex = 1;

        private float gameTimeOffset = 0;

        private Transition transDir = Transition.None;

        public MainMenu()
            : base()
        {
            _menuItems = RhinoLoader.GetMenuItems("LoadFiles\\ContentLoad.properties", "LoadFiles\\MainMenu.lfi");
            StateManager.Instance.Push(GameState.MainMenu);
            _inputBinding = new InputBinding(Keyboard.GetState(), Mouse.GetState());

            _focusIndex = 1;
        }

        public override void Update(GameTime gameTime)
        {
            if (StateManager.Instance.Peek() == GameState.MainMenu)
            {
                if (transDir != Transition.None)
                {
                    TransitionMenu(transDir);
                }
                else
                {
                    foreach (IMenuItem mi in _menuItems)
                    {
                        mi.Update(gameTime);
                    }

                    if (_inputBinding.CheckKeyDown(Keyboard.GetState(), Keys.Enter))
                    {
                        switch (_focusIndex)
                        {
                            case 1:
                                StateManager.Instance.Push(GameState.InGame);
                                RhinoXNAAdapter.Instance.RemoveAllMenus();
                                break;
                            case 2:
                            //    break;
                            //case 3:
                                TransitionMenu(Transition.Right);
                                break;
                            case 3:
                                TransitionMenu(Transition.Left);
                                break;
                            default:
                                StateManager.Instance.Push(GameState.ForceQuit);
                                break;
                        }
                    }

                    if (_inputBinding.CheckKeyDown(Keyboard.GetState(), Keys.S) && _focusIndex < 4)
                    {
                        //Move MM-Focus
                        IMenuItem item = _menuItems.Find(delegate(IMenuItem mi) { return mi.Sprite.Handle == "MM-Focus"; });
                        item.Sprite.Position = new Microsoft.Xna.Framework.Vector2(item.Sprite.Position.X, item.Sprite.Position.Y + 100.0f);
                        _focusIndex++;
                    }

                    if (_inputBinding.CheckKeyDown(Keyboard.GetState(), Keys.W) && _focusIndex > 1)
                    {
                        //Move MM-Focus
                        IMenuItem item = _menuItems.Find(delegate(IMenuItem mi) { return mi.Sprite.Handle == "MM-Focus"; });
                        item.Sprite.Position = new Microsoft.Xna.Framework.Vector2(item.Sprite.Position.X, item.Sprite.Position.Y - 100.0f);
                        _focusIndex--;
                    }

                    _inputBinding.Update();
                }
            }
        }

        private void TransitionMenu(Transition trans)
        {
            transDir = trans;

            switch (transDir)
            {
                case Transition.Left:
                    if (gameTimeOffset >= -400)
                    {
                        foreach (IMenuItem mi in _menuItems.FindAll(delegate(IMenuItem mi) { return mi.Sprite.Handle != "nebula"; }))
                        {
                            mi.Sprite.Position = new Vector2(mi.Sprite.Position.X - 20, mi.Sprite.Position.Y);
                        }

                        gameTimeOffset -= 20;
                    }
                    else
                    {
                        transDir = Transition.Center;
                        RhinoXNAAdapter.Instance.AddNewMenu(new CreditsMenu());
                    }
                    break;
                case Transition.Right:
                    if (gameTimeOffset <= 400)
                    {
                        foreach (IMenuItem mi in _menuItems.FindAll(delegate(IMenuItem mi) { return mi.Sprite.Handle != "nebula"; }))
                        {
                            mi.Sprite.Position = new Vector2(mi.Sprite.Position.X + 20, mi.Sprite.Position.Y);
                        }

                        gameTimeOffset += 20;
                    }
                    else
                    {
                        transDir = Transition.Center;
                        RhinoXNAAdapter.Instance.AddNewMenu(new HelpMenu());
                    }
                    break;
                case Transition.Center:
                    if (gameTimeOffset != 0)
                    {
                        if (gameTimeOffset <= 0)
                        {
                            foreach (IMenuItem mi in _menuItems.FindAll(delegate(IMenuItem mi) { return mi.Sprite.Handle != "nebula"; }))
                            {
                                mi.Sprite.Position = new Vector2(mi.Sprite.Position.X + 20, mi.Sprite.Position.Y);
                            }

                            gameTimeOffset += 20;
                        }
                        else if (gameTimeOffset >= 0)
                        {
                            foreach (IMenuItem mi in _menuItems.FindAll(delegate(IMenuItem mi) { return mi.Sprite.Handle != "nebula"; }))
                            {
                                mi.Sprite.Position = new Vector2(mi.Sprite.Position.X - 20, mi.Sprite.Position.Y);
                            }

                            gameTimeOffset -= 20;
                        }
                    }
                    else
                    {
                        transDir = Transition.None;
                    }
                    break;
            }
        }

        public override void Render()
        {
            foreach (IMenuItem mi in _menuItems)
            {
                mi.Render();
            }
        }
    }
}
